#include <iostream>
#include <GL/glew.h>
#include <kt84/glut_util.h>
#include <boost/lexical_cast.hpp>
#include <igl/read.h>
#include "measure_backfacingness.h"
using namespace std;
using namespace Eigen;

int main(int argc, char* argv[]) {
    auto print_usage = [&] () {
        cerr << "Usage: " << argv[0] << " [option(s)] input_mesh.{off|obj}\n"
             << "Options:\n"
             << "\t-r=<int> \t\t Framebuffer resolution (default:1024)\n"
             << "\t-d       \t\t Write debug images as png\n"
             << "Output in stdout: \t A real number in [0, 1] representing the ratio of pixels belonging to back-facing polygons in all the rendered pixels"
             << endl;
        return 1;
    };
    
    if (argc < 2)
        return print_usage();
    
    // Parse options
    int framebuffer_resolution = 1024;
    bool debug = false;
    for (int i = 1; i < argc - 1; ++i) {
        string op = argv[i];
        string op_head = op.substr(0, 2);
        if (op_head == "-r") {
            if (op[2] != '=') {
                cerr << "Wrong format: " << op << endl;
                return print_usage();
            }
            framebuffer_resolution = boost::lexical_cast<int>(op.substr(3, op.length() - 3));
        
        } else if (op_head == "-d") {
            debug = true;
        
        } else {
            cerr << "Unrecognized option: " << op << endl;
            return print_usage();
        }
    }
    
    string filename = argv[argc - 1];
    MatrixXd V;
    MatrixXi F;
    if (!igl::read(filename, V, F)) {
        cerr << "Failed to open: " << filename << endl;
        return 1;
    }
    
    // Init OpenGL
    glutInit(&argc, argv);
    glutCreateWindow("");
    glewInit();
    
    cout << measure_backfacingness(V, F, framebuffer_resolution, debug);
}
